#include <sparkconfig.h>
#include <spark/spark.h>
#include <zeitgeist/zeitgeist.h>
#include <oxygen/simulationserver/simulationserver.h>
#include <plugin/plugin.h>

#include "traincontrol.h"
#include "train/trainserver.h"
#include "train/trainsystem.h"
#include "utils/network.h"
#include "utils/trajectory.h"
#include "utils/kinematics.h"
#include "agent/worldmodel.h"
#include "agent/behavior.h"
#include "agent/tactic.h"
#include "agent/oppmodel.h"
#include "skill/kick.h"
#include "skill/block.h"
#include "skill/getup.h"
#include "skill/walkto.h"
#include "agent/tactic.h"
#include "agent/oppmodel.h"

unsigned int gUnum = 0;

using namespace spark;
using namespace plugin;
using namespace oxygen;
using namespace zeitgeist;
using namespace salt;
using namespace std;
using namespace boost;

class SimSpark : public Spark
{
public:
    SimSpark(const std::string& relPathPrefix) :
        Spark(relPathPrefix),
        mPlugin(new plugin::Plugin(*mZeitgeist))
    {};

    /** called once after Spark finished it's init */
    virtual bool InitApp(int argc, char** argv);

    /** print command line options */
    void PrintHelp();

    /** print a greeting */
    void PrintGreeting();

    /** process command line options */
    bool ProcessCmdLine(int argc, char* argv[]);

    /** run simspark.rb */
    void RunSimSparkRbScript();

protected:
    boost::shared_ptr<plugin::Plugin> mPlugin;
};

void SimSpark::PrintGreeting()
{
    //GetLog()->Normal()
    cout
        << "-------------------------------------------\n"
        << "-     The RoboCup Simulation 3D League    -\n"
        << "-                                         -\n"
        << "-             Team Apollo3D               -\n"
        << "-                                         -\n"
        << "- Copyright (C) 2007, 2008  Apollo3D Team -\n"
        << "- Junqing Wang, Ben, Ke Ding, Yikun Huang -\n"
        << "-------------------------------------------\n";
        //<< "\nType '--help' for further information\n\n";
}

void SimSpark::PrintHelp()
{
    GetLog()->Normal()
        << "\nusage: apollo3d [options]\n"
        << "\noptions:\n"
        << " --help\t print this message.\n"
        << "\n";
}

bool SimSpark::ProcessCmdLine(int argc, char* argv[])
{
    for( int i = 0; i < argc; i++)
    {
        if( strcmp( argv[i], "--help" ) == 0 )
        {
            PrintHelp();
            return false;
        }
    }

    return true;
}

void SimSpark::RunSimSparkRbScript()
{
    const string agentnum = "1";
    string host;
    int port;
    GetScriptServer()->GetVariable("Agent.Host", host);
    GetScriptServer()->GetVariable("Agent.Port", port);

    shared_ptr<SimulationServer> ss = shared_dynamic_cast<SimulationServer>
        (GetCore()->Get(gServerPath+"simulation"));
    ss->EnableStep(false);

    GetCore()->CreateContext()->New(gTeamName + "/TrainServer",
                                    gServerPath + "trainserver");

    GetCore()->CreateContext()->New(gTeamName + "/WorldModel",
                                    gServerPath + gTeamName + agentnum + "/worldmodel");
    GetCore()->CreateContext()->New(gTeamName + "/OppModel",
                                    gServerPath + gTeamName + agentnum + "/oppmodel");
    GetCore()->CreateContext()->New(gTeamName + "/Tactic",
                                    gServerPath + gTeamName + agentnum + "/tactic");
    GetCore()->CreateContext()->New(gTeamName + "/Behavior",
                                    gServerPath + gTeamName + agentnum + "/behavior");

    shared_ptr<Network> network = shared_dynamic_cast<Network>
        (GetCore()->CreateContext()->New(gTeamName + "/Network",
                                         gServerPath + gTeamName + agentnum + "/network"));
    network->SetServer(host);
    network->SetPort(port);

    GetCore()->CreateContext()->New(gTeamName + "/Trajectory",
                                    gServerPath + gTeamName + agentnum + "/trajectory");
    GetCore()->CreateContext()->New(gTeamName + "/Kinematics",
                                    gServerPath + gTeamName + agentnum + "/kinematics");
    GetCore()->CreateContext()->New(gTeamName + "/WalkTo",
                                    gServerPath + gTeamName + agentnum + "/walkto");
    GetCore()->CreateContext()->New(gTeamName + "/Kick",
                                    gServerPath + gTeamName + agentnum + "/kick");
    GetCore()->CreateContext()->New(gTeamName + "/GetUp",
                                    gServerPath + gTeamName + agentnum + "/getup");
    GetCore()->CreateContext()->New(gTeamName + "/Block",
                                    gServerPath + gTeamName + agentnum + "/block");

    ss->InitControlNode(gTeamName + "/TrainControl", "traincontrol" + agentnum);
}

bool SimSpark::InitApp(int argc, char** argv)
{
    PrintGreeting();

    // process command line
    if (! ProcessCmdLine(argc, argv))
    {
        return false;
    }

    // run initialization scripts
    GetScriptServer()->Run("apollo3d.rb");
    
    // control
    GetCore()->RegisterClassObject(new CLASS(TrainControl),     gTeamName + "/");

    // agent
    GetCore()->RegisterClassObject(new CLASS(WorldModel),       gTeamName + "/");
    GetCore()->RegisterClassObject(new CLASS(Behavior),         gTeamName + "/");
    GetCore()->RegisterClassObject(new CLASS(Tactic),           gTeamName + "/");
    GetCore()->RegisterClassObject(new CLASS(OppModel),         gTeamName + "/");

    // train
    GetCore()->RegisterClassObject(new CLASS(TrainServer),      gTeamName + "/");
    GetCore()->RegisterClassObject(new CLASS(TrainSystem),      "/");

    // utils
    GetCore()->RegisterClassObject(new CLASS(Network),          gTeamName + "/");
    GetCore()->RegisterClassObject(new CLASS(Trajectory),       gTeamName + "/");
    GetCore()->RegisterClassObject(new CLASS(Kinematics),       gTeamName + "/");
    
    // skill
    GetCore()->RegisterClassObject(new CLASS(Action),           gTeamName + "/");
    GetCore()->RegisterClassObject(new CLASS(Walk),             gTeamName + "/");
    GetCore()->RegisterClassObject(new CLASS(WalkTo),           gTeamName + "/");
    GetCore()->RegisterClassObject(new CLASS(Kick),             gTeamName + "/");
    GetCore()->RegisterClassObject(new CLASS(GetUp),            gTeamName + "/");
    GetCore()->RegisterClassObject(new CLASS(Block),            gTeamName + "/");

    RunSimSparkRbScript();

    return true;
}

int main(int argc, char** argv)
{
    // the spark app framework instance
    SimSpark spark("../../");
    if (! spark.Init(argc, argv))
    {
        return 1;
    }

    spark.GetSimulationServer()->SetSimStep(0.02);
    spark.GetSimulationServer()->Run(argc,argv);

    return 0;
}

/** multi-threads test code
shared_ptr<Behavior>            gBehavior;
shared_ptr<TrainControl>        gTC;
shared_ptr<Network>             gNet;
shared_ptr<WorldModel>          gWM;

int
main(int argc, char* argv[])
{
    // the spark app framework instance
    SimSpark spark("../../");
    if (! spark.Init(argc, argv))
    {
        return 1;
    }

    spark.GetSimulationServer()->SetSimStep(0.02);

    string num = "1";
    gBehavior = shared_dynamic_cast<Behavior>
        (spark.GetCore()->Get(gServerPath + gTeamName + num + "/behavior"));
    gTC = shared_dynamic_cast<TrainControl>
        (spark.GetCore()->Get(gServerPath + "simulation/traincontrol" + num));
    gNet = shared_dynamic_cast<Network>
        (spark.GetCore()->Get(gServerPath + gTeamName + num + "/network"));
    gWM = shared_dynamic_cast<WorldModel>
        (spark.GetCore()->Get(gServerPath + gTeamName + num + "/worldmodel"));

    gTC->InitSimulation();
    Run();
    gTC->DoneSimulation();
}
*/

